// Compact Networking Utilities (see devplanet.fastethernet.net)
// provided under the GNU Public License (see file COPYING)
// (C) 1998 Patrick Lambert <drow@darkelf.net>     

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;
// GWE SQL JDBC driver (see README)
import java.sql.*;
import gwe.sql.*;
// Telnet class (see README)
import display.*;
import socket.*;
import modules.*;

class CNU implements ActionListener, ItemListener
{
 // variables
 public boolean DEBUG = true;
 private String version = "0.1";    // *** PRODUCT VERSION ***
 private static String CONFIG_FILE = "CNU.cfg";
 Frame mainWin = null, cfgBox = null, textWin = null, buttonWin = null, infoWin = null;
 Choice action = null;
 Frame dialogWin = null;
 TextField t1 = null, t2 = null, t3 = null, t4 = null, t5 = null, t6 = null, t7 = null;
 TextArea textBox = null;
 List mail_list = null;
 BufferedReader in = null;
 PrintWriter out = null;
 Socket sockfd = null;
 Checkbox checkBox = null;
 // settings
 private String USERNAME = "noname";
 private String PASSWORD = "";
 private String OUT_SERVER = "localhost";
 private String POP_SERVER = "mail";
 private String EMAIL = "noname@localhost";
 private String SIGNATURE = "";
 private String TEXT_EDITOR = "internal";
 private int X_LOCATION = 50, Y_LOCATION = 50;

 public void print_debug(String line)
 {
  Generic f = new Generic();
  f.DISABLE_CONTROL_BOX = true;
  if(DEBUG) f.texteditor(null, line, "Debug information", false, 450, 300);
 }

 private void settings()
 {
  cfgBox = new Frame("Settings");
  Button ok = new Button("Save");
  Button cancel = new Button("Cancel");
  ok.addActionListener(this);
  cancel.addActionListener(this);
  cfgBox.addWindowListener(new WinControl());
  cfgBox.setSize(300, 270);
  cfgBox.setLocation(100, 150);
  cfgBox.setLayout(new GridLayout(8, 2));
  cfgBox.add(new Label("User name:", Label.CENTER));
  t1 = new TextField(USERNAME, 15);
  cfgBox.add(t1);
  cfgBox.add(new Label("Your e-mail:", Label.CENTER));
  t2 = new TextField(EMAIL, 15);
  cfgBox.add(t2);
  cfgBox.add(new Label("X Y location:", Label.CENTER));
  t3 = new TextField(X_LOCATION + " " + Y_LOCATION, 15);
  cfgBox.add(t3);
  cfgBox.add(new Label("POP3 incoming server:", Label.CENTER));
  t6 = new TextField(POP_SERVER, 15);
  cfgBox.add(t6);
  cfgBox.add(new Label("Send mail server:", Label.CENTER));
  t4 = new TextField(OUT_SERVER, 15);
  cfgBox.add(t4);
  cfgBox.add(new Label("Text editor:", Label.CENTER));
  t7 = new TextField(TEXT_EDITOR, 15);
  cfgBox.add(t7);
  cfgBox.add(new Label("Enable debugging:", Label.CENTER));
  if(DEBUG)
  t5 = new TextField("true", 15);
  else
  t5 = new TextField("false", 15);
  cfgBox.add(t5);
  cfgBox.add(ok);
  cfgBox.add(cancel);
  cfgBox.show();
 }

 public void itemStateChanged(ItemEvent evt)
 {
  Generic f = new Generic();
  f.DISABLE_CONTROL_BOX = true;
  if("  About".equals(evt.getItem()))
   f.texteditor(null, "About the Compact Networking Utilities v" + version + "\n\nThis program was made with 2 main goals: portability and compactness. It can be very useful for software developers, systems admins or power users to have direct access to common protocols.\n\nThis program is provided under the GNU Public General License (see file COPYING)\n\nCopyright 1998 Patrick Lambert <drow@darkelf.net>\n\nThis product includes third parties software (see file README)\n\nYou can get the latest copy at devplanet.fastethernet.net\n", "Help: About", false, 450, 300);
  if("  Quit the program".equals(evt.getItem()))
   System.exit(0);
  if("  Change settings".equals(evt.getItem()))
   settings();
  if("  System information".equals(evt.getItem()))
   sys_info();
  if("  Text editor".equals(evt.getItem()))
  {
   if(("internal".equals(TEXT_EDITOR)))
   {
    f.DISABLE_CONTROL_BOX = false;
    f.texteditor("noname.txt", "", "Editor", true, 600, 400);
   }
   else
   {
    Runtime rt = Runtime.getRuntime();
    try
    {
     rt.exec(TEXT_EDITOR);
    }
    catch(Exception err)
    {
     f.msgbox("Could not run process", "Error", true);
     print_debug(err.getMessage());
    }
   }
  }
  if("  Change password".equals(evt.getItem()))
   ask_password();
  if("  Send mail (SMTP)".equals(evt.getItem()))
   compose_mail();
  if("  Host information".equals(evt.getItem()))
   host_info();
  if("  User information".equals(evt.getItem()))
   user_info();
  if("  Domain information".equals(evt.getItem()))
   domain_info();
  if("  User manual".equals(evt.getItem()))
  {
   String result = "";
   result += "User guide:\n\nHere is a description of all the commands available in this program:\n\n";
   result += "Change settings: You can change your settings here, including your user name, e-mail address, and mail servers. The changes will be saved in the file Mail.cfg.\n\n";
   result += "Change password: This will change your password for this session only. For security issues the password is not saved in Mail.cfg. If you don't want to enter your password every time and you are on a single-user platform, you can add \"password: your_password ;\" in Mail.cfg\n\n";
   result += "Quit the program: You can quit the program with this and close any window currently open within this program.\n\n";
   result += "System information: This will pop up a window displaying system information.\n\n";
   result += "Text editor: This will launch the internal text editor or an external one you specified in the Settings.\n\n";
   result += "Host/User/Domain information: This will query a host to get information about a user, a hostname or a domain name.\n\n";
   result += "Check e-mails (POP3): Using the POP3 protocol, the program will connect to your POP3 server and tell you if you have new mail.\n\n";
   result += "Send mail: This simply asks you to compose a new single-part e-mail and will send it to the addresses you want.\n\n";
   result += "Telnet session: This command will open a telnet session window and allow you to connect to remote Unix server.\n\n";
   result += "Send a query (SQL): This command will use the SQL protocol to send a query to a database and return raw information.\n\n";
   result += "This program is (C) 1998 Patrick Lambert. See the About menu choice.\n\n";
   f.texteditor(null, result, "Help", false, 450, 300);
  }
  if("  Check e-mails (POP3)".equals(evt.getItem()))
  {
   mainWin.setTitle("Querying...");
   pop_list();
   mainWin.setTitle("Ready.");
  }
  if("  Read a message (POP3)".equals(evt.getItem()))
   entry_box("E-mail number (from 1 to n) to fetch", "Get e-mail", false, false);
  if("  Delete a message (POP3)".equals(evt.getItem()))
   entry_box("E-mail number (from 1 to n) to delete", "Delete e-mail", false, false);
  if("  Send a query (SQL)".equals(evt.getItem()))
   sql_link();
  if("  Run binary".equals(evt.getItem()))
   entry_box("Enter binary file you want to run", "Run it", false, false);
  if("  Telnet session".equals(evt.getItem()))
   entry_box("Syntax: hostname port", "Start telnet", false, false);
 }

 public void actionPerformed(ActionEvent evt)
 {
  Generic f = new Generic();
  String line = "";
  boolean state = false;
  if(evt.getActionCommand().equals("Cancel"))
  {
   cfgBox.dispose();
  }
  if(evt.getActionCommand().equals("Cancel message"))
  {
   buttonWin.dispose();
   textWin.dispose();
  }
  if(evt.getActionCommand().equals("Send message"))
  {
   mainWin.setTitle("Sending...");
   send_mail();
   buttonWin.dispose();
   textWin.dispose();
   mainWin.setTitle("Ready.");
  }
  if(evt.getActionCommand().equals("Save"))
  {
   mainWin.setTitle("Saving...");
   USERNAME = t1.getText();
   EMAIL = t2.getText();
   TEXT_EDITOR = t7.getText();
   X_LOCATION = Integer.parseInt(f.lindex(t3.getText(), 0));
   Y_LOCATION = Integer.parseInt(f.lindex(t3.getText(), 1));
   OUT_SERVER = t4.getText();
   POP_SERVER = t6.getText();
   if("true".equals(t5.getText())) DEBUG = true;
   else DEBUG = false;
   cfgBox.dispose();
   save_settings();
   mainWin.setTitle("Ready.");
  }
  if(evt.getActionCommand().equals("Save password"))
  {
   PASSWORD = t2.getText();
   dialogWin.dispose();
  }
  if(evt.getActionCommand().equals("Query SQL db"))
  {
   query_sql("jdbc:" + t1.getText(), t2.getText(), t3.getText(), t4.getText());
  }
  if(evt.getActionCommand().equals("Check host"))
  {
   mainWin.setTitle("Querying...");
   line = t2.getText();
   state = checkBox.getState();
   dialogWin.dispose();
   get_host_info(line, state);
   mainWin.setTitle("Ready.");
  }
  if(evt.getActionCommand().equals("Check user"))
  {
   mainWin.setTitle("Querying...");
   line = t2.getText();
   dialogWin.dispose();
   get_user_info(line);
   mainWin.setTitle("Ready.");
  }
  if(evt.getActionCommand().equals("Check domain"))
  {
   mainWin.setTitle("Querying...");
   line = t2.getText();
   dialogWin.dispose();
   get_domain_info(line);
   mainWin.setTitle("Ready.");
  }
  if(evt.getActionCommand().equals("Get e-mail"))
  {
   mainWin.setTitle("Querying...");
   line = t2.getText();
   dialogWin.dispose();
   get_pop_mail(line, false);
   mainWin.setTitle("Ready.");
  }
  if(evt.getActionCommand().equals("Delete e-mail"))
  {
   mainWin.setTitle("Querying...");
   line = t2.getText();
   dialogWin.dispose();
   get_pop_mail(line, true);
   mainWin.setTitle("Ready.");
  }
  if(evt.getActionCommand().equals("Run it"))
  {
   Runtime rt = Runtime.getRuntime();
   try
   {
    rt.exec(t2.getText());
   }
   catch(Exception err)
   {
    f.msgbox("Could not run process", "Error", true);
    print_debug(err.getMessage());
   }
   dialogWin.dispose();
  }
  if(evt.getActionCommand().equals("Start telnet"))
  {
   try
   {
    telnet applet = new telnet();
    applet.params = new Hashtable();
    applet.params.put("port", f.lindex(t2.getText(), 1));
    applet.params.put("address", f.lindex(t2.getText(), 0));
    applet.params.put("VTscrollbar", "true");
    Frame frame = new Frame("The Java Telnet Application");
    frame.setLayout(new BorderLayout());
    frame.add("Center", applet);
    frame.setSize(380, 590);
    frame.addWindowListener(new WinControl());
    applet.init();
    frame.pack();
    frame.show();
    applet.start();
   } catch(Exception err)
   {
    f.msgbox("An error occured during the session", "Error", false);
   }
  }
  mainWin.requestFocus();
  mainWin.toFront();
 }

 private void sql_link()
 {
  cfgBox = new Frame("SQL query");
  Button ok = new Button("Query SQL db");
  Button cancel = new Button("Cancel");
  ok.addActionListener(this);
  cancel.addActionListener(this);
  cfgBox.addWindowListener(new WinControl());
  cfgBox.setSize(400, 200);
  cfgBox.setLocation(100, 150);
  cfgBox.setLayout(new GridLayout(5, 2));
  cfgBox.add(new Label("URL:", Label.CENTER));
  t1 = new TextField("mysql://localhost:3306/database_name", 15);
  cfgBox.add(t1);
  cfgBox.add(new Label("Username:", Label.CENTER));
  t2 = new TextField("", 15);
  cfgBox.add(t2);
  cfgBox.add(new Label("Password:", Label.CENTER));
  t3 = new TextField("", 15);
  t3.setEchoChar('*');
  cfgBox.add(t3);
  cfgBox.add(new Label("SQL query:", Label.CENTER));
  t4 = new TextField("", 15);
  cfgBox.add(t4);
  cfgBox.add(ok);
  cfgBox.add(cancel);
  cfgBox.show();
 }

 public void query_sql(String url, String user, String pass, String query)
 {
  String result = "";
  int i;
  Generic f = new Generic();
  try
  {
   Class.forName("exgwe.sql.gweMysqlDriver");
   Connection con = DriverManager.getConnection(url, user, pass);
   DatabaseMetaData dmd = con.getMetaData();
   result += "Results from database:\n\n";
   result += "Driver name: " + dmd.getDriverName() + "\n";
   result += "Database product name: " + dmd.getDatabaseProductName() + "\n";
   Statement stmt = con.createStatement();
   ResultSet rs = stmt.executeQuery(query);
   ResultSetMetaData rsmd = rs.getMetaData();
   result += "Column count: " + rsmd.getColumnCount() + "\n\n";
   while(rs.next())
   {
    for(i = 1; i <= rsmd.getColumnCount(); i++)
     result += rs.getString(i) + " ";
    result += "\n";
   }
   rs.close();
   stmt.close();
   con.close();
  } catch(Exception err)
  {
   result += "Error contacting database: " + err.getMessage() + "\n\n";
  }
  f.DISABLE_CONTROL_BOX = true;
  f.texteditor(null, result, "Database info", false, 450, 300);
 }

 public void sys_info()
 {
  Runtime rt = Runtime.getRuntime();
  Generic f = new Generic();
  String result = "";
  result += "Operating system: " + System.getProperty("os.name") + "\n";
  result += "Version: " + System.getProperty("os.version") + "\n";
  result += "Architecture: " + System.getProperty("os.arch") + "\n";
  result += "User: " + System.getProperty("user.name") + " @ " + System.getProperty("user.home") + "\n";
  result += "Java VM: " + System.getProperty("java.version") + " @ " + System.getProperty("java.home") + "\n";
  result += "Java VM vendor: " + System.getProperty("java.vendor") + " @ " + System.getProperty("java.vendor.url") + "\n";
  result += "Total memory in JVM: " + rt.totalMemory() + "\n";
  result += "Free memory: " + rt.freeMemory() + "\n";
  f.DISABLE_CONTROL_BOX = true;
  f.texteditor(null, result, "System information", false, 450, 300);
 }

 private void host_info()
 {
  entry_box("Which hostname do you want to query?", "Check host", false, true);
 }

 private void user_info()
 {
  entry_box("Enter a user@hostname to query", "Check user", false, false);
 }

 private void domain_info()
 {
  entry_box("Which domain do you want to query?", "Check domain", false, false);
 }

 public void get_domain_info(String domain)
 {
  String line = "", result = "";
  Generic f = new Generic();
  try
  {
   sockfd = new Socket("rs.internic.net", 43);
   in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
   out = new PrintWriter(sockfd.getOutputStream(), true);
  }
  catch(Exception err)
  {
   f.msgbox("Can't connect to rs.internic.net", "Error", true);
   print_debug(err.getMessage());
   return;
  }
  try
  {
   out.println(domain);
   while((line = in.readLine()) != null)
   {
    result += line + "\n";
   }
   f.DISABLE_CONTROL_BOX = true;
   f.texteditor(null, result, "Domain info", false, 450, 300);
  }
  catch(Exception err)
  {
   f.msgbox("Can't connect to rs.internic.net", "Error", true);
   print_debug(err.getMessage());
   return;
  }
 }

 public void get_user_info(String str)
 {
  String line = "", result = "";
  Generic f = new Generic();
  try
  {
   sockfd = new Socket(f.lindex(str, 1, "@"), 79);
   in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
   out = new PrintWriter(sockfd.getOutputStream(), true);
  }
  catch(Exception err)
  {
   f.msgbox("Can't connect to remote host", "Error", true);
   print_debug(err.getMessage());
   return;
  }
  try
  {
   out.println(f.lindex(str, 0, "@"));
   while((line = in.readLine()) != null)
   {
    result += line + "\n";
   }
   f.DISABLE_CONTROL_BOX = true;
   f.texteditor(null, result, "User info", false, 450, 300);
  }
  catch(Exception err)
  {
   f.msgbox("Can't connect to rs.internic.net", "Error", true);
   print_debug(err.getMessage());
   return;
  }
 }

 public void get_host_info(String host, boolean state)
 {
  Generic f = new Generic();
  String result = "Local host information:\n";
  String line = "";
  try
  {
   result += InetAddress.getLocalHost();
   result += "\n\n";
  }
  catch(Exception err)
  {
   result += "Can't get local hostname / IP.\n\n";
  }
  result += "Remote host information:\n";
  try
  {
   result += InetAddress.getByName(host);
   result += "\n\n";
  }
  catch(Exception err)
  {
   result += "Can't get remote hostname / IP.\n\n";
  }
  if(state)
  {
   long first, last;
   first = System.currentTimeMillis();
   try
   {
    sockfd = new Socket(host, 7);
    in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
    out = new PrintWriter(sockfd.getOutputStream(), true);
    sockfd.close();
   }
   catch(Exception err) { }
   last = System.currentTimeMillis();
   result += "\nPing delay: " + (last - first) + "ms\n\n";
   try
   {
    sockfd = new Socket(host, 80);
    in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
    out = new PrintWriter(sockfd.getOutputStream(), true);
    result += "A web server was found on that host:\n";
    out.println("HEAD / HTTP/1.0\n");
    sockfd.setTcpNoDelay(true);
    while((line = in.readLine()) != null) result += line + "\n";
    result += "\n\n";
    sockfd.close();
   }
   catch(Exception err)
   {
    result += "No web server is running on that host.\n\n";
   }
   try
   {
    sockfd = new Socket(host, 25);
    in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
    out = new PrintWriter(sockfd.getOutputStream(), true);
    result += "An SMTP mail server was found on that host:\n";
    result += f.lrange(in.readLine(), 1);
    result += "\n\n";
    sockfd.close();
   }
   catch(Exception err)
   {
    result += "No SMTP mail server is running on that host.\n\n";
   }
   try
   {
    sockfd = new Socket(host, 110);
    in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
    out = new PrintWriter(sockfd.getOutputStream(), true);
    result += "A POP3 mail server was found on that host:\n";
    result += f.lrange(in.readLine(), 1);
    result += "\n\n";
    sockfd.close();
   }
   catch(Exception err)
   {
    result += "No POP3 mail server is running on that host.\n\n";
   }
  }
  f.DISABLE_CONTROL_BOX = true;
  f.texteditor(null, result, "Host information", false, 450, 300);
 }

 public void pop_list()
 {
  String line;
  Generic f = new Generic();
  try
  {
   sockfd = new Socket(POP_SERVER, 110);
   in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
   out = new PrintWriter(sockfd.getOutputStream(), true);
  }
  catch(Exception err)
  {
   f.msgbox("Can't connect to POP server", "Error", true);
   print_debug(err.getMessage());
   return;
  }
  try
  {
   line = in.readLine();
   if(line.charAt(0) == '-') { pop_failed(line); return; }
   out.println("USER " + USERNAME);
   line = in.readLine();
   if(line.charAt(0) == '-') { pop_failed(line); return; }
   out.println("PASS " + PASSWORD);
   line = in.readLine();
   if(line.charAt(0) == '-') { pop_failed(line); return; }
   f.msgbox(f.lrange(line, 1), "Info", false);
   sockfd.close();
  }
  catch(IOException err)
  {
   f.msgbox("Error while talking to mail server", "Error", true);
   print_debug(err.getMessage());
  }
 }

 public void get_pop_mail(String number, boolean delete)
 {
  String line, result = "";
  Generic f = new Generic();
  try
  {
   sockfd = new Socket(POP_SERVER, 110);
   in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
   out = new PrintWriter(sockfd.getOutputStream(), true);
  }
  catch(Exception err)
  {
   f.msgbox("Can't connect to POP server", "Error", true);
   print_debug(err.getMessage());
   return;
  }
  try
  {
   line = in.readLine();
   if(line.charAt(0) == '-') { pop_failed(line); return; }
   out.println("USER " + USERNAME);
   line = in.readLine();
   if(line.charAt(0) == '-') { pop_failed(line); return; }
   out.println("PASS " + PASSWORD);
   line = in.readLine();
   if(line.charAt(0) == '-') { pop_failed(line); return; }
   if(delete)
   {
    out.println("DELE " + number);
    if(line.charAt(0) == '-') { pop_failed(line); return; }
   }
   else
   {
    out.println("RETR " + number);
    line = in.readLine();
    if(line.charAt(0) == '-') { pop_failed(line); return; }
    while(!".".equals(line = in.readLine()))
    {
     result += line + "\n";
    }
    f.DISABLE_CONTROL_BOX = true;
    f.texteditor(null, result, "E-mail #" + number, false, 450, 300);
   }
   sockfd.close();
  }
  catch(IOException err)
  {
   f.msgbox("Error while talking to mail server", "Error", true);
   print_debug(err.getMessage());
  }
 }

 public void pop_failed(String line)
 {
  Generic f = new Generic();
  f.msgbox("Error while talking to mail server", "Error", true);
  print_debug(line);
 }

 public void send_mail()
 {
  String line;
  Generic f = new Generic();
  mail_list = new List();
  try
  {
   sockfd = new Socket(OUT_SERVER, 25);
   in = new BufferedReader(new InputStreamReader(sockfd.getInputStream()));
   out = new PrintWriter(sockfd.getOutputStream(), true);
  }
  catch(Exception err)
  {
   f.msgbox("Can't connect to POP server", "Error", true);
   print_debug(err.getMessage());
   return;
  }
  try
  {
   line = in.readLine();
   if(line.charAt(0) == '5') { pop_failed(line); return; }
   out.println("HELO localhost");
   line = in.readLine();
   if(line.charAt(0) == '5') { pop_failed(line); return; }
   out.println("MAIL FROM: " + EMAIL);
   line = in.readLine();
   if(line.charAt(0) == '5') { pop_failed(line); return; }
   out.println("RCPT TO: " + t1.getText());
   line = in.readLine();
   if(line.charAt(0) == '5') { pop_failed(line); return; }
   if(!("".equals(t2.getText())))
   {
    out.println("RCPT TO: " + t2.getText());
    line = in.readLine();
    if(line.charAt(0) == '5') { pop_failed(line); return; }
   }
   if(!("".equals(t3.getText())))
   {
    out.println("RCPT TO: " + t3.getText());
    line = in.readLine();
    if(line.charAt(0) == '5') { pop_failed(line); return; }
   }
   out.println("DATA");
   line = in.readLine();
   if(line.charAt(0) == '5') { pop_failed(line); return; }
   out.println("X-Mailer: CNU");
   out.println("Subject: " + t5.getText());
   out.println(textBox.getText());
   out.println(".");
   sockfd.close();
  }
  catch(IOException err)
  {
   f.msgbox("Error while talking to mail server", "Error", true);
   print_debug(err.getMessage());
  }
 }

 private void compose_mail()
 {
  buttonWin = new Frame("Control box");
  textWin = new Frame("Compose text");
  textBox = new TextArea("\n\n"+SIGNATURE, 600, 400, TextArea.SCROLLBARS_VERTICAL_ONLY);
  Button send = new Button("Send message");
  Button canc = new Button("Cancel message");
  textWin.addWindowListener(new WinControl());
  buttonWin.addWindowListener(new WinControl());
  send.addActionListener(this);
  canc.addActionListener(this);
  textWin.setSize(600, 400);
  textWin.setLocation(100, 300);
  textWin.add(textBox);
  textBox.setEditable(true);
  textWin.show();
  buttonWin.setSize(400, 200);
  buttonWin.setLocation(100, 150);
  buttonWin.setLayout(new GridLayout(6, 2));
  buttonWin.add(new Label("To:", Label.CENTER));
  t1 = new TextField("", 15);
  buttonWin.add(t1);
  buttonWin.add(new Label("CC:", Label.CENTER));
  t2 = new TextField("", 15);
  buttonWin.add(t2);
  buttonWin.add(new Label("BCC:", Label.CENTER));
  t3 = new TextField("", 15);
  buttonWin.add(t3);
  buttonWin.add(new Label("From:", Label.CENTER));
  t4 = new TextField(EMAIL, 15);
  buttonWin.add(t4);
  buttonWin.add(new Label("Subject:", Label.CENTER));
  t5 = new TextField("", 15);
  buttonWin.add(t5);
  buttonWin.add(send);
  buttonWin.add(canc);
  buttonWin.show();
 }

 private void save_settings()
 {
  Generic f = new Generic();
  if(!("".equals(USERNAME)))
   f.filesave(CONFIG_FILE, "username: " + USERNAME + " ;", false);
  f.filesave(CONFIG_FILE, "location: " + X_LOCATION + " " + Y_LOCATION + " ;", true);
  if(!("".equals(OUT_SERVER)))
   f.filesave(CONFIG_FILE, "out_server: " + OUT_SERVER + " ;", true);
  if(!("".equals(POP_SERVER)))
   f.filesave(CONFIG_FILE, "pop_server: " + POP_SERVER + " ;", true);
  if(!("".equals(EMAIL)))
   f.filesave(CONFIG_FILE, "email: " + EMAIL + " ;", true);
  if(!("".equals(TEXT_EDITOR)))
   f.filesave(CONFIG_FILE, "text_editor: " + TEXT_EDITOR + " ;", true);
  if(DEBUG)
   f.filesave(CONFIG_FILE, "debug: true ;", true);
  else
   f.filesave(CONFIG_FILE, "debug: false ;", true);
 }

 private boolean readconfig()
 {
  Generic f = new Generic();
  String line = "";
  int c;
  try
  {
   InputStream fd = new FileInputStream(CONFIG_FILE);
   while((c=fd.read()) != -1)
   {
    if((char)c == '\n' || (char)c == '\0')
    {
      // check for known param
     if("username:".equals(f.lindex(line, 0))) USERNAME = f.lindex(line, 1);
     else if("text_editor:".equals(f.lindex(line, 0))) TEXT_EDITOR = f.lindex(line, 1);
     else if("password:".equals(f.lindex(line, 0))) PASSWORD = f.lindex(line, 1);
     else if("location:".equals(f.lindex(line, 0)))
     {
      X_LOCATION = Integer.parseInt(f.lindex(line, 1));
      Y_LOCATION = Integer.parseInt(f.lindex(line, 2));
     }
     else if("out_server:".equals(f.lindex(line, 0))) OUT_SERVER = f.lindex(line, 1);
     else if("pop_server:".equals(f.lindex(line, 0))) POP_SERVER = f.lindex(line, 1);
     else if("email:".equals(f.lindex(line, 0))) EMAIL = f.lindex(line, 1);
     else if("debug:".equals(f.lindex(line, 0)) && "true".equals(f.lindex(line, 1))) DEBUG = true;
     else if("debug:".equals(f.lindex(line, 0)) && "false".equals(f.lindex(line, 1))) DEBUG = false;
     else if("#".equals(f.lindex(line, 0))) {} // comments
     else print_debug("Warning: Unknown config line: " + line);
     line = "";
    }
    line += (char)c;
   }
  }
  catch(FileNotFoundException err)
  {
   print_debug("Warning: Config file not found");
   return false;
  }
  catch(java.io.IOException err)
  {
   print_debug("I/O exception reading config file");
   return false;
  }
  return true;
 }

 private void ask_password()
 {
  entry_box("Set the password for your account (" + USERNAME + ")", "Save password", true, false);
 }

 public void entry_box(String question, String label, boolean hide, boolean check)
 {
  dialogWin = new Frame("Question");
  Label theMessage = new Label(question, Label.CENTER);
  Button okButton = new Button(label);
  if(check) checkBox = new Checkbox("Show details", true);
  t2 = new TextField();
  if(hide) t2.setEchoChar('*');
  okButton.addActionListener(this);
  dialogWin.addWindowListener(new WinControl());
  if(check) dialogWin.setSize(question.length()*10, 160);
  else dialogWin.setSize(question.length()*10, 120);
  dialogWin.setLocation(100, 150);
  if(check) dialogWin.setLayout(new GridLayout(4, 1));
  else dialogWin.setLayout(new GridLayout(3, 1));
  dialogWin.add(theMessage);
  dialogWin.add(t2);
  if(check) dialogWin.add(checkBox);
  dialogWin.add(okButton);
  dialogWin.show();
 }

 private void makegui()
 {
  mainWin = new Frame("Ready.");
  action = new Choice();
  mainWin.addWindowListener(new Exit());
  action.addItemListener(this);
  mainWin.setSize(270, 60);
  action.addItem("*** Compact Networking Utilities ***");
  action.addItem("  Change settings");
  action.addItem("  Change password");
  action.addItem("  Quit the program");
  action.addItem("Generic utilities:");
  action.addItem("  System information");
  action.addItem("  Text editor");
  action.addItem("  Run binary");
  action.addItem("Network utilities:");
  action.addItem("  Host information");
  action.addItem("  User information");
  action.addItem("  Domain information");
  action.addItem("In:");
  action.addItem("  Check e-mails (POP3)");
  action.addItem("  Read a message (POP3)");
  action.addItem("  Delete a message (POP3)");
  action.addItem("Out:");
  action.addItem("  Send mail (SMTP)");
  action.addItem("  Telnet session");
  action.addItem("Database:");
  action.addItem("  Send a query (SQL)");
  action.addItem("Help:");
  action.addItem("  About");
  action.addItem("  User manual");
  mainWin.add(action);
  mainWin.setLocation(X_LOCATION, Y_LOCATION);
  mainWin.show();
  if("".equals(PASSWORD)) ask_password();
 }

 public static void main(String[] args)
 {
  CNU cmc = new CNU();
  if(args.length>1) CONFIG_FILE = args[1];
  cmc.readconfig();
  cmc.makegui();
 }
}

class Exit extends WindowAdapter implements WindowListener
{
 public void windowClosing(WindowEvent evt)
 {
  if(evt.getID() == WindowEvent.WINDOW_CLOSING) System.exit(0);
  else super.windowClosed(evt);
 }
}
